package at.gv.egiz.moazs.backend;

import at.gv.egiz.moazs.scheme.Marshaller;
import at.gv.egiz.moazs.scheme.MsgResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.concurrent.CompletableFuture;

import static java.util.concurrent.CompletableFuture.completedFuture;

@Component
public class LogResponseSink {

    private static final Logger log = LoggerFactory.getLogger(LogResponseSink.class);

    private final Marshaller msgMarshaller;

    public LogResponseSink(Marshaller msgMarshaller) {
        this.msgMarshaller = msgMarshaller;
    }

    /**
     * Log response to class logger at level=INFO.
     *
     * @param msgResponse
     * @return
     */
    public CompletableFuture<Void> log(MsgResponse msgResponse) {

        if(log.isInfoEnabled()) {
            var builder = new StringBuilder("Received the following Message: \n");
            builder.append(msgMarshaller.marshallXml(msgResponse.getResponseAsJAXBElement()));
            log.info(builder.toString());
        } else {
            log.warn("Will not log response because INFO level is disabled.");
        }

        return completedFuture(null);

    }
}
